@@ -25,6 +25,7 @@
 #include "grd-daemon.h"
 
 #include <gio/gio.h>
+#include <glib-unix.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <stdio.h>
@@ -41,6 +42,9 @@ struct _GrdDaemon
 {
   GApplication parent;
 
+  GSource *sigint_source;
+  GSource *sigterm_source;
+
   GCancellable *cancellable;
   guint remote_desktop_watch_name_id;
   guint screen_cast_watch_name_id;
@@ -74,7 +78,8 @@ start_rdp_server (GrdDaemon *daemon)
   GrdSettings *settings = grd_context_get_settings (daemon->context);
   g_autoptr (GError) error = NULL;
 
-  g_assert (!daemon->rdp_server);
+  if (daemon->rdp_server)
+    return;
 
   if (!g_access (grd_settings_get_rdp_server_cert (settings), F_OK) &&
       !g_access (grd_settings_get_rdp_server_key (settings), F_OK))
@@ -109,7 +114,8 @@ start_vnc_server (GrdDaemon *daemon)
 {
   g_autoptr (GError) error = NULL;
 
-  g_assert (!daemon->vnc_server);
+  if (daemon->vnc_server)
+    return;
 
   daemon->vnc_server = grd_vnc_server_new (daemon->context);
   if (!grd_vnc_server_start (daemon->vnc_server, &error))
@@ -369,6 +375,17 @@ grd_daemon_shutdown (GApplication *app)
 
   g_clear_object (&daemon->context);
 
+  if (daemon->sigterm_source)
+    {
+      g_source_destroy (daemon->sigterm_source);
+      g_clear_pointer (&daemon->sigterm_source, g_source_unref);
+    }
+  if (daemon->sigint_source)
+    {
+      g_source_destroy (daemon->sigint_source);
+      g_clear_pointer (&daemon->sigint_source, g_source_unref);
+    }
+
   G_APPLICATION_CLASS (grd_daemon_parent_class)->shutdown (app);
 }
 
@@ -399,6 +416,44 @@ add_actions (GApplication *app)
   g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (action));
 }
 
+static gboolean
+sigint_terminate_daemon (gpointer user_data)
+{
+  GrdDaemon *daemon = user_data;
+
+  g_debug ("Received SIGINT signal. Exiting...");
+  g_clear_pointer (&daemon->sigint_source, g_source_unref);
+  g_application_release (G_APPLICATION (daemon));
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+sigterm_terminate_daemon (gpointer user_data)
+{
+  GrdDaemon *daemon = user_data;
+
+  g_debug ("Received SIGTERM signal. Exiting...");
+  g_clear_pointer (&daemon->sigterm_source, g_source_unref);
+  g_application_release (G_APPLICATION (daemon));
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+register_signals (GrdDaemon *daemon)
+{
+  daemon->sigint_source = g_unix_signal_source_new (SIGINT);
+  g_source_set_callback (daemon->sigint_source, sigint_terminate_daemon,
+                         daemon, NULL);
+  g_source_attach (daemon->sigint_source, NULL);
+
+  daemon->sigterm_source = g_unix_signal_source_new (SIGTERM);
+  g_source_set_callback (daemon->sigterm_source, sigterm_terminate_daemon,
+                         daemon, NULL);
+  g_source_attach (daemon->sigterm_source, NULL);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -443,6 +498,7 @@ main (int argc, char **argv)
                       NULL);
 
   add_actions (app);
+  register_signals (GRD_DAEMON (app));
 
   settings = grd_context_get_settings (GRD_DAEMON (app)->context);
   if (rdp_port != -1)
