@@ -21,7 +21,9 @@
 #define GRD_PIPEWIRE_UTILS_H
 
 #include <gio/gio.h>
+#include <pipewire/pipewire.h>
 #include <stdint.h>
+#include <spa/param/video/format-utils.h>
 
 #include "grd-types.h"
 
@@ -38,10 +40,15 @@ typedef struct _GrdPipeWireSource
 
 void grd_maybe_initialize_pipewire (void);
 
+gboolean grd_pipewire_buffer_has_pointer_bitmap (struct pw_buffer *buffer);
+
+gboolean grd_pipewire_buffer_has_frame_data (struct pw_buffer *buffer);
+
 gboolean grd_spa_pixel_format_to_grd_pixel_format (uint32_t        spa_format,
                                                    GrdPixelFormat *out_format);
 
-void grd_sync_dma_buf (int      fd,
-                       uint64_t start_or_end);
+void grd_get_spa_format_details (enum spa_video_format  spa_format,
+                                 uint32_t              *drm_format,
+                                 int                   *bpp);
 
 #endif /* GRD_PIPEWIRE_UTILS_H */
